package circlePuzzles.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.BoxLayout;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import circlePuzzles.CirclePuzzles;
import circlePuzzles.data.Puzzle;
import circlePuzzles.grid.Grid;

public final class TabGrid extends MyTab
implements ActionListener, DocumentListener
{
   private PanelGrid designPanel;
   private JComboBox<Grid> gridList;
   private JButton zoomInButton, zoomOutButton;
   private JTextField description = new JTextField();

   public TabGrid (Puzzle puz){
      super(puz, "Grid", "Select the grid and circle size");

      description.getDocument().addDocumentListener(this);

      designPanel = new PanelGrid(puz);

      gridList = new JComboBox<Grid>(Grid.grids);
      gridList.setSelectedItem(puz.getGrid());
      gridList.addActionListener(this);

      zoomInButton = new JButton("+");
      zoomInButton.addActionListener(this);
      zoomOutButton = new JButton("-");
      zoomOutButton.addActionListener(this);

      JPanel controlPanel = new JPanel();
      controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));
      controlPanel.add(gridList);
      controlPanel.add(zoomOutButton);
      controlPanel.add(zoomInButton);

      JLabel text = new JLabel("Choose a grid below, then set circle size(s) by dragging them. Drag an active centre to add a concentric circle.");
      JPanel southPanel = new JPanel(new BorderLayout());
      southPanel.add(text,BorderLayout.WEST);
      southPanel.add(controlPanel,BorderLayout.SOUTH);

      setLayout(new BorderLayout());
      add(description, BorderLayout.NORTH);
      add(designPanel);
      add(southPanel, BorderLayout.SOUTH);
   }
   protected void init(){
      gridList.setSelectedItem(puz.getGrid());
      designPanel.init();
      description.setText(puz.getDescription());
   }
   protected void exit(){
      designPanel.exit();
   }

   public void initColours(){
      Color bgColour = CirclePuzzles.settingsManager.getColorSetting(CirclePuzzles.settingKeyBgColour);
      Color c1 = CirclePuzzles.settingsManager.getColorSetting(CirclePuzzles.settingKeyCircleColour1);
      Color c2 = CirclePuzzles.settingsManager.getColorSetting(CirclePuzzles.settingKeyCircleColour2);
      Color c3 = CirclePuzzles.settingsManager.getColorSetting(CirclePuzzles.settingKeyCircleColour3);
      Color c4 = CirclePuzzles.settingsManager.getColorSetting(CirclePuzzles.settingKeyCircleColour4);

      designPanel.setBackground(bgColour);
      designPanel.setCircleColours(c1, c2, c3, c4);
   }

   public void actionPerformed(ActionEvent ae){
      if      ( ae.getSource()==zoomInButton ){
         designPanel.zoomIn();
      }else if( ae.getSource()==zoomOutButton ){
         designPanel.zoomOut();
      }else if( ae.getSource()==gridList ){
         Grid g = (Grid) gridList.getSelectedItem();
         if( g!=puz.getGrid() ){
            designPanel.setGrid(g);
         }
      }
   }

   public void changedUpdate(DocumentEvent arg0) {
      puz.setDescription(description.getText());
   }
   public void insertUpdate(DocumentEvent arg0) {
      puz.setDescription(description.getText());
   }
   public void removeUpdate(DocumentEvent arg0) {
      puz.setDescription(description.getText());
   }
}